% Code connected to the publication
% Time-shifted cost function design for more efficient dynamic wind farm flow control
% Author: M. Becker, D. Allaerts, J.W. van Wingerden

%% Define a wind farm layout
% n_t x 2 matrix with x,y coordinates in m
Layout =...
    [   0,  0;
      500,  0;
     1000,  0;
     1000,  500;
    ];
nT = size(Layout,1);    % Number of turbines (calculated)
D  = 174.8;             % Turbine diameter

%% Define flow conditions
U_inf       = 9;       % Free wind speed in m/s
wind_dir    = 270;     % Wind Direction in degrees

%% Define control settings
ah          = 30;      % Prescribed action horizon in time steps (no unit)
deltaT      = 5;       % Simulation time step in s

%% ///////////////// Run code \\\\\\\\\\\\\\\\\\\\\
[T,combined,ah,ph,solve_order] = ...
    calcT(Layout,nT,D,U_inf,wind_dir, ah, deltaT);
% OUTPUTS
% T_star  : (nO x nT*ph matrix) maps the power generated from all turbines
%           over the prediction horizon to nO optimisation const functions
% combined: (nO x nT*ah matrix) shows which action horizon steps are
%           optimised in which cost function
% ah      : (int) Action horizon in time steps [-]
% ph      : (int) Prediction horizon in time steps [-]
% solve_order : (1xnC cell array) collection of optimisation problems that
%               can be solved independently and if they depend on
%               oneanother then they are ordered by solve order

%% Plot T and parameter sets
figure
subplot(2,1,1)
imagesc(T)
title('Matrix T')
ylabel('Optimization problems')
xlabel('Outputs')

subplot(2,1,2)
imagesc(combined)
title("Optimization sets")
ylabel('Set')
xlabel('Optimization parameter')